from numpy import linspace, array
from matplotlib import pyplot as plt

from convergence_calculator import ConvergenceCalculator

# Example with three grids
# The calculator applies Roache's method
h1 = [0.1, 0.2, 0.4]
phi1 = [1.01, 1.04, 1.16]

c = ConvergenceCalculator(h1, phi1)

print('Result from three grid example:')
print('Extrapolated results is {:.3f}'.format(c.get_phi(0)))
print('The Grid Convergence Index is {:.3f}'.format(
    c.uncertainties[0]/abs(c.phi[0])
    ))
print('The apparent order of convergence is {:.3f}'.format(c.get_order()))


# Example with four grids
# The calculator applies the Eca/Hoekstra method
h2 = [0.1, 0.2, 0.4, 0.8]
phi2 = [1.01, 1.04, 1.16, 1.64]

c = ConvergenceCalculator(h2, phi2)

# Plot fitted function and input points
x_range = linspace(0, max(h2), 200)
p = plt.plot(x_range,  c.get_phi(x_range))
plt.plot(h2, phi2, 'x') # Input points

# Add errorbars for uncertainties
# Plot in the same color as the fitted function
color = p[0].get_color()
plt.errorbar(c.h, c.get_phi(array(c.h)), yerr = c.uncertainties,
        fmt = 'none', ecolor = color
        )

# Add title and axis labels
plt.title('Result from four grid example')
plt.xlabel('h')
plt.ylabel(r'$\phi$')
plt.show() # Open figure
