import unittest as ut

from convergence_calculator import ConvergenceCalculator

def rel(val, exact):
    return round(abs(val/exact), 3)

class TestConvergenceCalculator(ut.TestCase):
    def test_roache_1(self):
        h = [(1/n)**(1/2) for n in (18000, 8000, 4500)]
        phi = [6.063, 5.972, 5.863]
        c = ConvergenceCalculator(h, phi)
        self.assertEqual(
                [round(f, 3) for f in c.calc_refinement_factors()],
                [1.5, 1.333]
                )
        self.assertEqual(round(c.get_order(), 2), 1.53)
        self.assertEqual(round(c.get_phi(0), 4), 6.1685)
        self.assertEqual(rel(c.calc_approximate_errors()[0], phi[0]), 0.015)
        self.assertEqual(rel(c.calc_extrapolated_errors()[0], c.get_phi(0)), 0.017)
        self.assertEqual(rel(c.calc_gci()[0], phi[0]), 0.022)

    def test_roache_2(self):
        h = [(1/n)**(1/2) for n in (18000, 4500, 980)]
        phi = [10.7880, 10.7250, 10.6050]
        c = ConvergenceCalculator(h, phi)
        self.assertEqual(
                [round(f, 3) for f in c.calc_refinement_factors()],
                [2.0, 2.143]
                )
        self.assertEqual(round(c.get_order(), 2), 0.75)
        self.assertEqual(round(c.get_phi(0), 4), 10.8801)
        self.assertEqual(rel(c.calc_approximate_errors()[0], phi[0]), 0.006)
        self.assertEqual(rel(c.calc_extrapolated_errors()[0], c.get_phi(0)), 0.008)
        self.assertEqual(rel(c.calc_gci()[0], phi[0]), 0.011)

    def test_roache_3(self):
        h = [(1/n)**(1/2) for n in (18000, 4500, 980)]
        phi = [6.0042, 5.9624, 6.0909]
        c = ConvergenceCalculator(h, phi)
        self.assertEqual(
                [round(f, 3) for f in c.calc_refinement_factors()],
                [2.0, 2.143]
                )
        self.assertEqual(round(c.get_order(), 2), 1.51)
        self.assertEqual(round(c.get_phi(0), 4), 6.0269)
        self.assertEqual(rel(c.calc_approximate_errors()[0], phi[0]), 0.007)
        self.assertEqual(rel(c.calc_extrapolated_errors()[0], c.get_phi(0)), 0.004)
        self.assertEqual(rel(c.calc_gci()[0], phi[0]), 0.005)

    def test_eca_1st_order(self):
        h = [0.1, 0.2, 0.4, 0.8]
        phi = [1.1, 1.2, 1.4, 1.8]
        c = ConvergenceCalculator(h, phi)
        self.assertAlmostEqual(c.get_order(), 1)
        self.assertAlmostEqual(c.get_phi(0), 1)
        self.assertAlmostEqual(c.uncertainties[0], 0.125)

    def test_eca_2nd_order(self):
        h = [0.1, 0.2, 0.4, 0.8]
        phi = [1.1, 1.4, 2.6, 7.4]
        c = ConvergenceCalculator(h, phi)
        self.assertAlmostEqual(c.get_order(), 2)
        self.assertAlmostEqual(c.get_phi(0), 1)
        self.assertAlmostEqual(c.uncertainties[0], 0.125)

    def test_eca_half_order(self):
        h = [0.1, 0.2, 0.4, 0.8]
        phi = [1.1, 1.141421356, 1.2, 1.282842712]
        c = ConvergenceCalculator(h, phi)
        self.assertAlmostEqual(c.get_order(), 0.5)
        self.assertAlmostEqual(c.get_phi(0), 1)
        self.assertAlmostEqual(c.uncertainties[0], 0.125)
